@@ -90,6 +90,7 @@
 +(int64_t)readNumberInRangeFromBuffer:(NSRange)range buffer:(NSData *)buffer
 {
 	NSString *readString = [[NSString alloc] initWithData:[buffer subdataWithRange:range] encoding:NSASCIIStringEncoding];
+	if(!readString) return 0;
 	NSScanner* scanner = [NSScanner scannerWithString:readString];
 	[readString release];
 	long long returnValue;
@@ -455,6 +456,11 @@
 		// Read in the header
 		NSData *longHeader = [handle readDataOfLength:size];
 		[handle seekToFileOffset:offset];
+
+		// Check if there is a terminating null byte, and eliminate it.
+		int length=[longHeader length];
+		const uint8_t *bytes=[longHeader bytes];
+		if(length>0 && bytes[length-1]==0) longHeader=[longHeader subdataWithRange:NSMakeRange(0,length-1)];
 		
 		// Prepare a new dictionary with the next header.
 		NSData *header = [handle readDataOfLength:512];
